const path = require('path');

let Mocha = undefined;
try {
  Mocha = require('mocha');
} finally {
  // let Mocha be undefined in browser testing.  Handle it later
}
const expect = require('chai').expect;
const sinon = require('sinon');

const core = require('../varsnap/core');

const testCasePath = path.resolve(__dirname, '..', 'varsnap', 'test.js');

function example(x, y) {
  return x * y;
}

function SilentReporter(runner) {
  Mocha.reporters.Base.call(this, runner);
}

core.varsnap.config = {
  varsnap: 'true',
  env: 'development',
  consumerToken: 'consumer-abcd',
};

describe('TestCase', () => {
  beforeEach(function() {
    core.Util.resetProducers();
    core.Util.resetConsumers();

    if(Mocha === undefined) {
      this.skip();
    }

    this.mocha = new Mocha({
      reporter: SilentReporter,
    });
    this.mocha.addFile(testCasePath);
    this.mocha.unloadFiles();
  });

  it('will consume', function(done) {
    core.varsnap(example);

    const consumers = core.Util.getConsumers();
    expect(consumers.length).to.equal(1);
    const consumer = consumers[0];
    const stub = sinon.stub(consumer, 'consumeOnce');
    stub.resolves(true);

    this.mocha.run((failures) => {
      expect(consumer.consumeOnce.called).to.be.true;
      expect(failures).to.equal(0);
      done();
    });
  });

  it('will error out if consume fails', function(done) {
    core.varsnap(example);

    const consumers = core.Util.getConsumers();
    expect(consumers.length).to.equal(1);
    const consumer = consumers[0];
    const stub = sinon.stub(consumer, 'consumeOnce');
    stub.resolves('asdf');

    this.mocha.run((failures) => {
      expect(consumer.consumeOnce.called).to.be.true;
      expect(failures).to.equal(1);
      done();
    });
  });
});
